{****************************************************************************
**
*W  commentator_pas.pas    Homology source                  DHS
**
*H  @(#)$Id: commentator_pas.pas,v 0.0 1999/08
**
*Y  Copyright (C)  1999-2003,  Us guys
**
**  This file implements the Pascal interface to commentators (for
**  providing runtime commentary to the user)

** commentator_pas.pas header for Pascal code corresponding to commentator.h for C++
*}

{$gnu-pascal,I+}

{$if __GPC_RELEASE__ >= 20030303}
{$ifdef COMMENTATOR_STANDALONE}
{$define NAME(X) attribute (name = X)}
{$else}
{$define NAME(X) external name X}
{$endif}
{$else}
{$ifdef COMMENTATOR_STANDALONE}
{$define NAME(X) asmname X}
{$else}
{$define NAME(X) external; asmname X}
{$endif}
{$endif}

unit Commentator_Pas;

interface

type
  TCommentator = Pointer;

const
  { Min and Max }
  { initializing commentator with PrintEverything, everything should be printed. }
  { initializing commentator with PrintNothing, nothing should be printed. }
  PrintEverything = 100000;
  PrintNothing = 0;

  { Msg Class }
  TimingMeasure	= 1;
  TimingEstimate	= 2;
  PartialResult	= 3;
  Internal_Warning	= 4;
  Internal_Error	= 5;
  Internal_Description = 6;

  { Msg Level }
  LvlAlways = -1;
  LvlImp    = 0;
  LvlNormal = 1;
  LvlUnimp  = 2;
  LvlBlabla = 10;

function InitializeCommentator(Timing,	
			       Homology	: Integer): TCommentator;
NAME ('initializeCommentator');
{ Create a commentator object reference. }

procedure StartActivity(Com : TCommentator;
			Id,
			Msg: CString;
			MsgLevel,
			MsgClass: Integer);
NAME ('startActivity');
{ Start an activity, indentation level will be increased,
  printlevel = activity depth + MsgLevel. }

procedure StopActivity(Com : TCommentator;
		       Msg : CString;
		       MsgLevel,
		       MsgClass: Integer);
NAME ('stopActivity');
{ Stop an activity, indentation level will be decreased. }

procedure ActivityReport(Com: TCommentator;
			 Msg: CString;
			 MsgLevel,
			 MsgClass: Integer);
NAME ('activityReport');
{ Provides a message to the commentary object C for possible printing/display. }

procedure ProgressReport(Com: TCommentator;
			 Msg: CString;
			 MsgLevel,
			 k, n: Integer);
NAME ('progressReport');
{ Provides a message to the commentary object assuming k out of n steps of the activity have been completed. }
{ This allows the Commentator to make time estimates. }

function IsPrinted(Com: TCommentator;
 	   MsgLevel,
		   MsgClass: Integer): Integer;
NAME ('isPrinted');
{ Returns 0 if messages of this level and class will not be printed,
  non-zero if they will be printed. }

implementation

{$ifdef COMMENTATOR_STANDALONE}

uses GPC, Tools;

const
  CR = #13;

var
  Activity: TString = '';

procedure StartLine;
begin
  if MsgStep <> 0 then Write (' ' : 79, CR)
end;

function InitializeCommentator;
var Dummy: Integer;
begin
  Dummy := Timing;
  Dummy := Homology;
  InitializeCommentator := nil
end;

procedure StartActivity;
var
  Dummy1: TCommentator;
  Dummy2: CString;
  Dummy3: Integer;
begin
  Dummy1 := Com;
  Dummy2 := Msg;
  Dummy3 := MsgLevel;
  Dummy3 := MsgClass;
  Activity := CString2String (Id);
  if Activity = 'elimination' then Activity := 'elim.'
end;

procedure StopActivity;
var
  Dummy1: TCommentator;
  Dummy2: Integer;
begin
  Dummy1 := Com;
  Dummy2 := MsgLevel;
  Dummy2 := MsgClass;
  StartLine;
  WriteLn (CString2String (Msg))
end;

procedure ActivityReport;
var Dummy: TCommentator;
begin
  Dummy := Com;
  if (MsgLevel <= LvlImp) or (MsgStep <> 0) then
    begin
      StartLine;
      Write (CString2String (Msg));
      if (MsgLevel <= LvlNormal) or (MsgClass <> Internal_Description) then
        WriteLn
      else
        Write ('. ');
      if MsgLevel <= LvlAlways then Flush (Output)
    end
end;

procedure ProgressReport;
var Dummy: TCommentator;
begin
  Dummy := Com;
  if (MsgStep <> 0) and ((MsgLevel <= LvlUnimp) or (k mod MsgStep = 0)) then
    Write (Activity, ': ', k, '/', n, ' ', CString2String (Msg), CR)
end;

function IsPrinted;
var
  Dummy1: TCommentator;
  Dummy2: Integer;
begin
  Dummy1 := Com;
  Dummy2 := MsgClass;
  IsPrinted := Ord (MsgLevel <= LvlUnimp)
end;

{$endif}

end.
